local mod = HowMany:NewModule("HMLoot", "AceHook-3.0", "AceEvent-3.0" )
local L = LibStub("AceLocale-3.0"):GetLocale("HowMany")
mod.modName = L["HMLoot"]
mod.canBeDisabled = true;

-- Find my parser module.  If it's not there, then, well, I can't do anything
local ParseMod = HowMany:GetModule("Parser")


local UnitName = _G.UnitName
local pairs = _G.pairs
local select = _G.select
local type = _G.type
local gsub = _G.string.gsub
local string_find = string.find
local ChatFrame_GetMessageEventFilters = _G.ChatFrame_GetMessageEventFilters
local HMDebPrint = HowMany.DebugPrint


-- Money strings.  Take copies of the global strings, remove the parameters.  This gives us, essentially, translated words for Gold, silver, copper
local GOLD = gsub(GOLD_AMOUNT, "%%d ", "")
local SILVER = gsub(SILVER_AMOUNT, "%%d ", "")
local COPPER = gsub(COPPER_AMOUNT, "%%d ", "")


local player = UnitName("player")
local defaults = {
	profile = {
		trackMoney = true,
		moneyAsWords = false,
		moneyAsColors = true,
		trackItems = true
	}
}

local options = {
	type = "group",
	name = L["HMLoot Options"],
	order = 1,
	childGroups = 'tree',
	args = {
		Enable = {
			type = "toggle", 
			name = L["Enable "] .. "module HMLoot",
			desc = L["Enable "] .. "module HMLoot",
			order = 11,
			get = function()
				return HowMany.db.profile.modules["HMLoot"] ~= false or false
			end,
			set = function(_, v)
				HowMany.db.profile.modules["HMLoot"] = v
				if v then
					HowMany:EnableModule("HMLoot")											
				else
					HowMany:DisableModule("HMLoot")					
				end
			end
		},
		MoneyOptions = {
			type = "group",
			name = L["Money Loot Options"],
			order = 1,
			args = {
				money = {
					type = "toggle",
					name = L["Show Total Money"],
					desc = L["Show total money on loot messages"],
					descStyle = "inline",
					order = 5,
					get = function()
						return mod.db.profile.trackMoney
					end,
					set = function(info, v)
						mod.db.profile.trackMoney = v
					end
				},
				moneywords = {
					type = "toggle",
					name = L["Show with words"],
					desc = L["Show denomination with words.  321 Gold 45 Silver 67 Copper"],
					descStyle = "inline",
					order = 7,
					get = function()
						return mod.db.profile.moneyAsWords
					end,
					set = function(info, v)
						mod.db.profile.moneyAsWords = v
					end
				},
				moneycolor = {
					type = "toggle",
					name = L["Colorize money loots"],
					desc = L["Show total money using colors. |cffffd700321|r |cff80808045|r |cffeda55f67|r"],
					descStyle = "inline",
					order = 8,
					get = function()
						return mod.db.profile.moneyAsColors
					end,
					set = function(info, v)
						mod.db.profile.moneyAsColors = v
					end
				}
			}
		},
		ItemOptions = {
			type = "group",
			name = L["Item Loot Options"],
			order = 2,
			args = {			
				items = {
					type = "toggle",
					name = L["Show Bags Total"],
					desc = L["Show total in bags on loot messages"],
					descStyle = "inline",
					order = 6,
					get = function()
						return mod.db.profile.trackItems
					end,
					set = function(info, v)
						mod.db.profile.trackItems = v
					end
				}
			}
		}
	}
}

-- These are the events that I'm interested in parsing.
local valid_events = {
	CHAT_MSG_LOOT = true,
	CHAT_MSG_MONEY = true
}

-- *********************************************************
-- *********************************************************
function mod:OnInitialize()
	self.db = HowMany.db:RegisterNamespace("HMLoot", defaults)
end


--mod.stringToAdd = ""

-- *********************************************************
-- *********************************************************
local function HandleMoney(parserEvent)
--HMDebPrint("In HandleMoney.." .. parserEvent.moneyString )

if ( not mod.db.profile.trackMoney ) then return end

local mstart, mend, capCopper, capSilver, capGold

-- pull the amount of gold, silver, copper in this loot message
mstart, mend, capCopper = string_find(parserEvent.moneyString,"(%d+) " .. COPPER)
capCopper = capCopper or 0
mstart, mend, capSilver = string_find(parserEvent.moneyString,"(%d+) " .. SILVER)
capSilver = capSilver or 0
mstart, mend, capGold = string_find(parserEvent.moneyString,"(%d+) " .. GOLD)
capGold = capGold or 0

-- Add what was just looted to the player total
 local playerMoney = GetMoney() + capCopper + 100*capSilver + 100*100*capGold
 local playerGold = math.floor(playerMoney / (100 * 100))
 playerMoney = playerMoney - 100*100 * playerGold
 local playerSilver = math.floor(playerMoney / 100)
 local playerCopper = playerMoney % 100

-- Format the result
  if ( mod.db.profile.moneyAsWords ) then
      playerGold = tostring(playerGold) .. " " .. GOLD
      playerSilver = tostring(playerSilver) .. " " .. SILVER
      playerCopper = tostring(playerCopper) .. " " .. COPPER
  end
 
  if ( mod.db.profile.moneyAsColors ) then
  	parserEvent.resultString = "|cffffd700" .. playerGold .. "|r |cff808080" .. playerSilver .. "|r " .. "|cffeda55f" .. playerCopper .. "|r"
  else
    parserEvent.resultString = playerGold .. " " .. playerSilver .. " " .. playerCopper
  end
 
end


-- *********************************************************
-- *********************************************************
local function HandleItems(parserEvent)
--HMDebPrint("In HandleItems" )
 -- Created items are subject to a race condition.  Sometimes they're already in inventory when we get notified,
 -- sometimes they're not.  So just ignore them.
 if (parserEvent.isCreate) then return end

 -- Get information about the looted item.
 local itemLink = parserEvent.itemLink
 
 -- Get the number of items already existing in inventory and add the amount
 -- looted to it if the item wasn't the result of a conjure.
 local numLooted = parserEvent.amount or 1
 local numItems = GetItemCount(itemLink) or 0  
 local numTotal = numItems + numLooted
 
 parserEvent.resultString = " You now have " .. numTotal .. "."

end


-- *********************************************************
-- *********************************************************
local function ParserEventsHandler(parserEvent,msgText)
  
 -- Ignore the event if it isn't for the player or not a loot event.
 if (parserEvent.recipientUnit ~= "player" or parserEvent.eventType ~= "loot") then return end

 -- Call the correct handler for the loot type.
 if (parserEvent.isMoney) then HandleMoney(parserEvent) elseif (parserEvent.itemLink) then HandleItems(parserEvent) end
end


-- *********************************************************
-- *********************************************************
function mod:OnEnable()
	
	HMDebPrint("In HMLoot:OnEnable" )
	
	-- Set the raw hooks on the chat frames.  Raw hooks allow us to
	-- intercept the message and possibly modify it before it reaches
	-- its intended destination (the chat frame)
	for i = 1, NUM_CHAT_WINDOWS do
		local cf = _G["ChatFrame" .. i]
		if cf ~= COMBATLOG then
			self:RawHook(cf, "AddMessage", true)
			--HMDebPrint("Hooked " .. i )
		end
	end
	
end


-- *********************************************************
-- *********************************************************
function mod:OnDisable()
end




-- *********************************************************
-- *********************************************************
function mod:AddMessage(frame, text, ...)
	if not text then 
		return self.hooks[frame].AddMessage(frame, text, ...)
	end

	if valid_events[event] and type(arg2) == "string" then
		
		if ( ParseMod ~= nil ) then
			
			local pEvent = ParseMod.ParseRawChatEvent(event,text)
			
			if ( pEvent ) then
				ParserEventsHandler(pEvent,text)
				if ( pEvent.resultString ) then
					text = text .. pEvent.resultString
				end
			end
		end
	end 	
		
	return self.hooks[frame].AddMessage(frame, text, ...)
end

-- *********************************************************
-- Present some information to Ace
-- *********************************************************
function mod:Info()
	return L["When looting money or items, show how much/many you now have."]
end

function mod:GetOptions()
	return options
end

